@@ -138,14 +138,14 @@ module Vips
           image = image.cast :float
         end
 
-        new_format = image.format == :double ? :dpcomplex : :complex
+        new_format = (image.format == :double) ? :dpcomplex : :complex
         image = image.copy format: new_format, bands: image.bands / 2
       end
 
       image = block.call(image)
 
       unless Image.complex? original_format
-        new_format = image.format == :dpcomplex ? :double : :float
+        new_format = (image.format == :dpcomplex) ? :double : :float
         image = image.copy format: new_format, bands: image.bands * 2
       end
 
@@ -458,7 +458,14 @@ module Vips
       loader = Vips.vips_foreign_find_load_source source
       raise Vips::Error if loader.nil?
 
-      Vips::Operation.call loader, [source], opts, option_string
+      result = Vips::Operation.call loader, [source], opts, option_string
+
+      # keep a secret ref to the source object ... the libvips loader will
+      # keep a ref to the C source object, but we need the ruby wrapper object
+      # to stay alive too
+      result.references << source
+
+      result
     end
 
     def self.matrix_from_array width, height, array
