@@ -49,10 +49,10 @@ module Vips
 
     def initialize name
       name = name.to_s if name.is_a? Symbol
-      ptr = Vips.vips_interpolate_new name
-      raise Vips::Error if ptr.nil?
+      pointer = Vips.vips_interpolate_new name
+      raise Vips::Error if pointer.nil?
 
-      super ptr
+      super(pointer)
     end
   end
 end
