@@ -1,3 +1,11 @@
+# Set default concurrency so we can check against it later. Must be set
+# before Vips.init sets concurrency to the default.
+DEFAULT_VIPS_CONCURRENCY = 5
+ENV["VIPS_CONCURRENCY"] = DEFAULT_VIPS_CONCURRENCY.to_s
+
+# Disable stderr output since we purposefully trigger warn-able behavior.
+ENV["VIPS_WARNING"] = "1"
+
 require "vips"
 
 require "tempfile"
@@ -14,6 +22,14 @@ def timg(name)
   File.join(@temp_dir, name)
 end
 
+def has_jpeg?
+  Vips.type_find("VipsOperation", "jpegload") != nil
+end
+
+def has_svg?
+  Vips.type_find("VipsOperation", "svgload") != nil
+end
+
 RSpec.configure do |config|
   config.around do |example|
     Dir.mktmpdir("ruby-vips-spec-") do |dir|
